"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.License = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const fs_1 = require("fs");
const utility_marker_1 = require("../utility.marker");
const logger_1 = require("../logger");
function License(STATE) {
    return __awaiter(this, void 0, void 0, function* () {
        const CONFIG = yield STATE.getConfigSection('license');
        if (!CONFIG)
            return;
        const license = yield (yield node_fetch_1.default(`https://api.github.com/licenses/${CONFIG.type}`)).json();
        const OUT = CONFIG.out || 'README.md';
        const FILE = CONFIG.file || 'LICENSE';
        const YEAR = CONFIG.year ? CONFIG.year : new Date().getFullYear();
        const input = utility_marker_1.readFileAndAddMarker(OUT, 'license');
        const readmeText = `Copyright (c) YEAR NAME Licensed under the TYPE license.`
            .replace(/NAME/, CONFIG.name)
            .replace(/YEAR/, YEAR.toString())
            .replace(/TYPE/, license.spdx_id);
        yield fs_1.promises.writeFile(OUT, input.replace(utility_marker_1.insertionMarker.regex, utility_marker_1.insertGenerated(readmeText, 'license')));
        logger_1.log('info', logger_1.genMessage('License'), OUT);
        yield fs_1.promises.writeFile(FILE, license.body.replace(/\[year\]/, YEAR).replace(/\[fullname\]/, CONFIG.name));
        logger_1.log('info', logger_1.genMessage('License'), FILE);
    });
}
exports.License = License;
