"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TsDoc = void 0;
const suf_node_1 = require("suf-node");
const fs_1 = require("fs");
const path_1 = require("path");
const utility_marker_1 = require("../utility.marker");
const logger_1 = require("../logger");
const codeBlock = '```';
function TsDoc(STATE) {
    return __awaiter(this, void 0, void 0, function* () {
        const section = yield STATE.getConfigSection('tsDoc');
        if (section === undefined)
            return;
        const CONFIG = addDefaultsToConfig(section);
        const filesPaths = yield getPaths(CONFIG);
        const input = utility_marker_1.readFileAndAddMarker(CONFIG.out, 'tsDoc');
        const declarationBlockRegex = /(\/\*\*?[\S\s]*?\*\/\n)?export[ \t]*(declare|interface)[ \t]*([\w-]*)[ \t]*([\w-]*)(.*?;|[\S\s]*?^})/gm;
        let output = '';
        const navLinks = [];
        for (const path of filesPaths) {
            const fileText = fs_1.readFileSync(path).toString();
            const fileName = path.replace(/.*(\/|\\\\)([\w\.-]*)\.d\.ts/, '$2');
            let match;
            navLinks.push(`#${fileName}`);
            let fileOutput = `\n### ${fileName}\n`;
            while ((match = declarationBlockRegex.exec(fileText)) !== null) {
                /* istanbul ignore if */
                if (match.index === declarationBlockRegex.lastIndex) {
                    declarationBlockRegex.lastIndex++;
                }
                let [all, comment, declaration, type, name, content] = match;
                const isInternalDeclaration = /^internal/i.test(convertComment(comment));
                if (!isInternalDeclaration) {
                    navLinks.push(declaration === 'interface' ? type : name);
                    fileOutput += `\n##### ${declaration === 'interface' ? type : name}\n
${codeBlock}ts
${all.replace(/export ?| ?declare ?/g, '').replace(/\n^[ \t]*private .*$/gm, '')}
${codeBlock}
`;
                }
            }
            output += fileOutput;
        }
        const title = CONFIG.title ? `\n# ${CONFIG.title}\n` : '';
        yield fs_1.promises.writeFile(CONFIG.out, input.replace(utility_marker_1.insertionMarker.regex, utility_marker_1.insertGenerated(`${title}${createNav(navLinks)}${output}`, 'tsDoc')));
        logger_1.log('info', logger_1.genMessage('TsDoc'), CONFIG.out);
    });
}
exports.TsDoc = TsDoc;
function addDefaultsToConfig(config) {
    return Object.assign(Object.assign({}, config), { dir: /* istanbul ignore next */ (config === null || config === void 0 ? void 0 : config.dir) || '', out: /* istanbul ignore next */ (config === null || config === void 0 ? void 0 : config.out) || 'README.md' });
}
function getPaths(CONFIG) {
    return __awaiter(this, void 0, void 0, function* () {
        const dir = yield suf_node_1.Walk(path_1.resolve('./', CONFIG.dir));
        if (CONFIG.exclude !== undefined && CONFIG.include !== undefined) {
            logger_1.log('error', '[suf-cli:tsDoc] Cannot use option "include" and "exclude" at the same time, all d.ts files will be used.');
            return dir.filter((fileName) => fileName.endsWith('d.ts'));
        }
        const isInclude = CONFIG.include !== undefined ? CONFIG.include.length === 0 : true;
        const type = isInclude ? 'exclude' : 'include';
        const checkArr = CONFIG[type] || [];
        const includeCheck = (a, b) => a === b;
        const excludeCheck = (a, b) => a !== b;
        const check = isInclude ? includeCheck : excludeCheck;
        const filename = (name) => path_1.basename(name).replace(/\.d\.ts$/, '');
        const filesPaths = dir.filter((fileName) => fileName.endsWith('d.ts') && check(checkArr.indexOf(filename(fileName)), -1));
        return filesPaths;
    });
}
function createNav(links) {
    let output = '';
    for (const link of links) {
        if (link.startsWith('#')) {
            output += `\n- **[${link.replace(/#/, '')}](${link.toLowerCase()})**\n\n`;
        }
        else {
            output += `  - [${link}](#${link.toLowerCase()})\n`;
        }
    }
    return output;
}
function convertComment(comment) {
    return comment ? comment.replace(/(\/?\*\*?\/?|[\t \n]*)/g, '') : '';
}
